@@ -1,6 +1,7 @@
 # Modified 25 Nov 2009 for point marking
 # 20 Jan 2010: changed line types. J. Fox
 # 15 August 2010: fixed colors of points
+# 18 January 2011; added robust M estimation
 
 invTranPlot <- function(x,...) UseMethod("invTranPlot")
 
@@ -25,21 +26,22 @@ invTranPlot.formula <- function(x, data, subset, na.action, ...) {
          invTranPlot(vx,vy,...)
   }   
 
-invTranPlot.default<- function(x, y, lambda=c(-1, 0, 1), 
+invTranPlot.default<- function(x, y, lambda=c(-1, 0, 1), robust=FALSE, 
         lty.lines=rep(c("solid", "dashed", "dotdash", "longdash", "twodash"), 
         length=1 + length(lambda)), lwd.lines=2, 
         col=palette()[1], col.lines=palette(), 
         xlab=deparse(substitute(x)), ylab=deparse(substitute(y)),
         family="bcPower", optimal=TRUE, key="auto",
-        id.method = abs(residuals(lm(y~x))),
+        id.method = "x",
         labels, 
         id.n = if(id.method[1]=="identify") Inf else 0, 
         id.cex=1, id.col=palette()[1], grid=TRUE, ...){
  if (missing(labels)) labels <- seq(length(x))
  if (is.factor(x)) stop("Predictor variable may not be a factor")
  if (is.factor(y)) stop("Response variable may not be a factor")
- if (optimal){opt <- invTranEstimate(x, y, family=family, confidence=FALSE)
-              lam <- c(opt$lambda, lambda)} else lam <- lambda
+ if (optimal){
+     opt <- invTranEstimate(x, y, family=family, confidence=FALSE, robust=robust)
+     lam <- c(opt$lambda, lambda)} else lam <- lambda
  fam <- match.fun(family)
  plot(x, y, xlab=xlab, ylab=ylab, type="n", col=col, ...)
  if(grid){
@@ -49,8 +51,8 @@ invTranPlot.default<- function(x, y, lambda=c(-1, 0, 1),
  rss <- NULL
  new <- seq(min(x, na.rm=TRUE), max(x,na.rm=TRUE), length=100)
  for (j in 1:length(lam)){
-     m1 <- lm(y~fam(x, lam[j]))
-     rss <- c(rss, deviance(m1))
+     m1 <- if(robust) rlm(y ~ fam(x, lam[j])) else lm(y~fam(x, lam[j]))
+     rss <- c(rss, sum(residuals(m1)^2))
      lines(new,predict(m1, data.frame(x=new)), lty=lty.lines[j],
        col=col.lines[j], lwd=lwd.lines)}
  showLabels(x, y, labels=labels, 
@@ -76,14 +78,18 @@ invTranPlot.default<- function(x, y, lambda=c(-1, 0, 1),
             inset=0.02,  cex=0.75, fill=c("#00000000",col.lines), 
             border=c("#00000000",col.lines))
       }}
- data.frame(lambda=lam,RSS=rss)
+ data.frame(lambda=lam, RSS=rss)
 }
 
-invTranEstimate <- function(x,y,family="bcPower",confidence=0.95){
+invTranEstimate <- function(x, y, family="bcPower", confidence=0.95,
+  robust=FALSE){
   if (is.factor(x)) stop("Predictor variable may not be a factor")
   if (is.factor(y)) stop("Response variable may not be a factor")
+  if (robust) confidence <- FALSE
   fam <- match.fun(family)
-  f <- function(lambda,x,y,family){deviance(lm(y~fam(x,lambda)))}
+  f <- if(robust==FALSE)
+    function(lambda,x,y,family){deviance(lm(y~fam(x,lambda)))}  else
+    function(lambda,x,y,family){sum(residuals(rlm(y ~ fam(x,lambda)))^2)}
   lhat <- optimize(f = function(lambda) f(lambda,x,y,family),interval=c(-10,10))
   if (confidence==FALSE){ return(list(lambda=lhat$minimum)) } else {
     g <- lm(y~fam(x,lhat$minimum))
